#!/bin/ksh

# =======================================================================
#
#                   Jet Propulsion Laboratory
#
#                  Atmospheric InfraRed Sounder
#
#      Copyright (c) 2000 - California Institute of Technology
#              U.S. Government Sponsorship under NASA
#                Contract NAS7-1270 is acknowledged
# !PROLOG ==================================================================
#
# !Routine/File Name:
#
#       L1B_AIRS.ksh
#
#
# !Description:
#
#    This is a script to run the Level-1B AIRS PGE
#
#    It repeatedly calls the level-1B AIRS executable for each granule
#    of data.
#
#    It returns 0 (success) unless all granules fail.
#
#
# !Input Parameters:
#
#    Number of granules to process is read in from the PCF file
#    (specified by $PGS_PC_INFO_FILE) from Logical ID (LID) 1022
#
# !Output Parameters:
#
#    None
#
# !Input/Output Parameters:
#
#    None
#
#
# !Global Variables:
#
#    None
#
#
# !Known Bugs and Limitations:
#
#    None
#
#
# !Revision History:  
#
#    Available through CCC/Harvest.
#
#
# !END ==================================================================

let numGranules=0
let numGranules=`PGS_PC_GetConfigDataCom 1022`
if [ numGranules -le 0 ]; then
   exit 1
fi
# echo  "# granules ="$numGranules

let item=1
let numFailures=0
while [ item -le $numGranules ]; do
   run_level_1B_airs $item
   let RESULT=$?
   run_retcode_msg Level-1B-IR-gran-$item-of-$numGranules $RESULT
   if [ $RESULT -ne 0 ]; then
      let numFailures=$numFailures+1
   fi
   let item=$item+1
done
# echo "numFailures ="$numFailures

# report number of failures to LogStatus
run_aggregate_final $numFailures $numGranules

# if numFailures < numGranules then at least one run succeeded.
if [ numFailures -ge $numGranules ]; then
   # echo "Reporting FAILURE"
   exit 1
fi
