#!/usr/bin/env python
#	!/opt/packages/python/bin/python

#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

#========================================================================
# File Name: airsTm.py
# Directory /home/rickf/Direct_Broadcast/bin/python/new_dev/Recovery
#
# Description:  airsTm.py is a collection of subroutines that manipulate
# time related values, converting to/from unix time strings and/or UTC
# formats.
#
# Developed as a component part of the AIRS Direct Broadcast system.
#========================================================================

import time, string, copy, calendar

EPOCH = 1970
def timegm(tuple):
    """Unrelated but handy function to calculate Unix timestamp from GMT."""
    year, month, day, hour, minute, second = tuple[:6]
    assert year >= EPOCH
    assert 1 <= month <= 12
    days = 365*(year-EPOCH) + calendar.leapdays(EPOCH, year)
    for i in range(1, month):
        days = days + calendar.mdays[i]
    if month > 2 and calendar.isleap(year):
        days = days + 1
    days = days + day - 1
    hours = days*24 + hour
    minutes = hours*60 + minute
    seconds = minutes*60 + second
    return seconds

# UTCTimeFormat is a format string for basic time conversion.  It does NOT
# deal with the 'NN.nnnnnnZ' seconds display in AIRS UTC time strings; that
# is separately handled.
STD_UTC_TIME_STR = '20:05:26.000000Z'
STD_UTC_DATE_STR =  '2001-09-16'
UTCTimeFormat="%Y-%m-%dT%H:%M:%S"
TimeTbl=[0]
ValidTbl = 0
BTime = timegm([2001, 10, 3, 0, 0, 0, 0, 0, 0])
BTime_6 = timegm([2001, 10, 3, 0, 6, 0, 0, 0, 0])
SixMinutes = BTime_6 - BTime

# Given a unix time, convert to string of the form YYDDDHHMMSS as
# a timehack value for use in file/directory names.
def timehack(ut):
   tmTpl = time.gmtime(ut)
   Yr = tmTpl[0]
   if (Yr >= 2000):
      Yr = Yr - 2000
   elif ((Yr < 2000) and (Yr >= 1900)):
      Yr = Yr - 1900
   #YR = "%02s" % Yr
   #YDAY = "%03s" % tmTpl[7]
   #HH = "%02s" % tmTpl[3]
   #MM = "%02s" % tmTpl[4]
   #SS = "%02s" % tmTpl[5]
   #strFmt = "%02s-%03s-%02s-%02s-%02s"
   strFmt = "%02d%03d%02d%02d%02d"
   datestr = strFmt % (Yr, tmTpl[7], tmTpl[3], tmTpl[4], tmTpl[5])
   return datestr


# Given a floating point value which represents a unix time (seconds since
# epoch), convert it to an AIRS standard UTC string of the form
# "YYYY-MM-DDTHH:MM:SS.ssssssZ"
def ut2UTC(timeVal):
   BaseTm = int(timeVal)
   BaseTpl = time.gmtime(timeVal)
   FractTm = timeVal-BaseTm
   FractTmStr = ("%2.6f" % FractTm)+'Z'
   FractTmStr = FractTmStr[1:]
   UTCdateStr = time.strftime(UTCTimeFormat, BaseTpl)+FractTmStr
   return UTCdateStr

# Given a python tuple which represents a unix tm data set plus
# seconds offset), convert it to an AIRS standard UTC string of the form
# "YYYY-MM-DDTHH:MM:SS.ssssssZ"
def tm2UTC(tmTuple):
   Tm = tmTuple[:9]
   FractTm = ("%2.6f" % tmTuple[9])+'Z'
   FractTm = FractTm[1:]
   UTCdateStr = time.strftime(UTCTimeFormat, Tm)
   UTCdateStr = UTCdateStr+FractTm
   return UTCdateStr


# converts the tuple to a standard unix time floating point value
def unixTime(tmTuple):
   if (len(tmTuple) > 9):
      tmtpl = tmTuple[0:-1]
      tmVal = timegm(tmtpl) + tmTuple[9]
   else:
      tmVal = timegm(tmTuple[0:-1])
   return tmVal

# convert an AIRS UTC time string in the format"YYYY-MM-DDTHH:MM:SS.SSSSSSZ" to a
# tuple.
def UTC2tm(tmStr):
   TimeCopy = copy.copy(tmStr)   #we don't want to modify the users UTC time string!
   Front, Back = string.split(TimeCopy, '.')
   Back = "0."+Back
   Back = string.replace(Back, 'Z', '')
   tmTpl = time.strptime(Front, UTCTimeFormat); #returns tuple for gmtime() fn.
   Seconds = string.atof(Back)
   tmTpl = tmTpl + (Seconds,)
   return tmTpl

# determine in which granule the passed UTC time string is positioned.
def getGranule(UTCtm):
   utime = unixTime(UTC2tm(UTCtm))
   Mark = len(TimeTbl)/2
   if ((utime > TimeTbl[Mark]) and (utime < TimeTbl[Mark+1])):
      return Mark
   if (utime < TimeTbl[Mark]):
      for idx in range (1,Mark):
         if ((utime > TimeTbl[idx]) and (utime < TimeTbl[idx+1])):
	    return idx
   else:
      for idx in range (Mark,len(TimeTbl)-1):
         if ((utime > TimeTbl[idx]) and (utime < TimeTbl[idx+1])):
	    return idx
   return -1

# get number of granules to process based on starttime, endtime
def getGranuleCount(ST,ET):
   SG = getGranule(ST)
   EG = getGranule(ET)
   return (EG-SG+1)


def getGranuleST(Gran):
   return TimeTbl[Gran]

def getGranuleET(Gran):
   ET=TimeTbl[Gran]+SixMinutes - 0.000001
   return ET


def isValidTbl():
   return ValidTbl

# Generate a Granule Start/End time table for the specified date.  If the date has
# not changed, just use the existing table.
def genStartTbl(DateStr, Granule):
   ValidTbl = 1
   PARTS = string.split(DateStr, 'T')
   PARTS[1] = STD_UTC_TIME_STR
   BaseTmStr = string.join (PARTS, 'T')
   BaseTime = UTC2tm(BaseTmStr)
   StartEpoch = unixTime(BaseTime) # set starting value
   BaseEpoch = unixTime(BaseTime) # set starting value

   # initialize the table
   for idx in range (len(TimeTbl),1200):
      TimeTbl.append(StartEpoch)

   for idx in range(0,Granule):
      Index = Granule-idx
      TimeTbl[Index]=float(StartEpoch)
      StartEpoch = StartEpoch-SixMinutes

   StartEpoch = BaseEpoch
   for idx in range(Granule,1200):
      TimeTbl[idx]=float(StartEpoch)
      StartEpoch = StartEpoch+SixMinutes
      
