
#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

# File Name: apid.py
# Directory: /home/rickf/Direct_Broadcast/bin/python/new_dev/Recovery

import string, os, airs_utils, airsTm

GBAD_APID = '957'

L1B_APIDS = ['6211', '6213', '6220', '6221',
	     '6222', '6223', '6230', '6231',
	     '6232', '6233', '7111', '7113',
	     '7120', '7121', '7122', '7123']

L1A_APIDS = ['257',  '259',  '260',  '261',  '262',  '288',  '289',  '290',
               '342',  '404',  '405',  '406',  '407',  '414',  '415',  '416',  '417',
               '9257', '9259', '9260', '9261', '9262', '9288', '9289', '9290',
               '9342', '9404', '9405', '9406', '9407', '9414', '9415', '9416', '9417']
PRIMARY_L1A_APIDS = ['257',  '259',  '260',  '261',  '262',  '288',  '289',  '290',
                       '342',  '404',  '405',  '406',  '407',  '414',  '415',  '416',  '417']

L1A_AMSU_APIDS = ['257',  '259',  '260',  '261',  '262',  '288',  '289',  '290',
              '9257', '9259', '9260', '9261', '9262', '9288', '9289', '9290']
PRIMARY_L1A_AMSU_APIDS = ['257',  '259',  '260',  '261',  '262',  '288',  '289',  '290']

L1A_HSB_APIDS = ['342', '9342']
PRIMARY_L1A_HSB_APIDS = ['342']

L1A_AIRS_APIDS = ['404',  '405',  '406',  '407',  '414',  '415',  '416',  '417',
              '9404', '9405', '9406', '9407', '9414', '9415', '9416', '9417']
PRIMARY_L1A_AIRS_APIDS = ['404',  '405',  '406',  '407',  '414',  '415',  '416',  '417']

L1B_AMSU_APIDS = ['6220', '6230', '7120']
PRIMARY_L1B_AMSU_APIDS = ['7120']
L1B_HSB_APIDS = ['6222', '6232', '7122']
PRIMARY_L1B_HSB_APIDS = ['7122']
L1B_AIRS_APIDS = ['6211', '6221', '7111', '6231', '7121']
PRIMARY_L1B_AIRS_APIDS = ['7111', '7121']
L1B_VIS_APIDS = ['6213', '6223', '7113', '6233', '7123']
PRIMARY_L1B_VIS_APIDS = ['7113', '7123']

InstNames = {'AMSU' : 0,
	     'AIRS' : 1,
             'HSB'  : 2,
	     'VIS'  : 3}

L1A_Instrument_Map = {'AIRS' : 'L1A-AIRS.pcf',
                      'AMSU' : 'L1A-AMSU.pcf',
		      'HSB'  : 'L1A-HSB.pcf'}
L1B_Instrument_Map = {'AIRS' : 'L1B-AIRS.pcf',
                      'AMSU' : 'L1B-AMSU.pcf',
		      'HSB'  : 'L1B-HSB.pcf',
		      'VIS'  : 'L1B-VIS.pcf'}

# MAP table format is key = APID based LID, data returned is Process Group,
# instrument, pcf template name.
L1A_APID_MAP = {'257' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '259' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '260' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '261' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '262' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '288' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '289' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '290' : ['AMSU', '0', 'L1A-AMSU.pcf'],
                '342' : ['HSB',  '2', 'L1A-HSB.pcf'],
                '404' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '405' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '406' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '407' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '414' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '415' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '416' : ['AIRS', '1', 'L1A-AIRS.pcf'],
                '417' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9257' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9259' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9260' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9261' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9262' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9288' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9289' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9290' : ['AMSU', '0', 'L1A-AMSU.pcf'],
               '9342' : ['HSB',  '2', 'L1A-HSB.pcf'],
               '9404' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9405' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9406' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9407' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9414' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9415' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9416' : ['AIRS', '1', 'L1A-AIRS.pcf'],
               '9417' : ['AIRS', '1', 'L1A-AIRS.pcf']}

L1B_APID_MAP = {'6211' : ['AIRS', '1', 'L1B-AIRS.pcf'],
		'6213' : ['VIS',  '3', 'L1B-VIS.pcf'],
		'6220' : ['AMSU', '0', 'L1B-AMSU.pcf'],
		'6221' : ['AIRS', '1', 'L1B-AIRS.pcf'],
		'6222' : ['HSB',  '2', 'L1B-HSB.pcf'],
		'6223' : ['VIS',  '3', 'L1B-VIS.pcf'],
		'6230' : ['AMSU', '0', 'L1B-AMSU.pcf'],
		'6231' : ['AIRS', '1', 'L1B-AIRS.pcf'],
		'6232' : ['HSB',  '2', 'L1B-HSB.pcf'],
		'6233' : ['VIS',  '3', 'L1B-VIS.pcf'],
		'7111' : ['AIRS', '1', 'L1B-AIRS.pcf'],
		'7113' : ['VIS',  '3', 'L1B-VIS.pcf'],
		'7120' : ['AMSU', '0', 'L1B-AMSU.pcf'],
		'7121' : ['AIRS', '1', 'L1B-AIRS.pcf'],
		'7122' : ['HSB',  '2', 'L1B-HSB.pcf'],
		'7123' : ['VIS',  '3', 'L1B-VIS.pcf']}

def getProcGroup(KEY):
   if (L1A_APID_MAP.has_key(KEY)):
      return L1A_APID_MAP[KEY][0]
   elif (L1B_APID_MAP.has_key(KEY)):
      return L1B_APID_MAP[KEY][0]
   else:
      return 'UNKNOWN'

def getInstVal(INSTRUMENT):
   return InstNames[INSTRUMENT]

def getInstrument(LID):
   if (L1A_APID_MAP.has_key(LID)):
      return L1A_APID_MAP[LID][1]
   elif (L1B_APID_MAP.has_key(LID)):
      return L1B_APID_MAP[LID][1]
   else:
      return 'UNKNOWN'

def getInstrumentName(LID):
   if (L1A_APID_MAP.has_key(LID)):
      return L1A_APID_MAP[LID][0]
   elif (L1B_APID_MAP.has_key(LID)):
      return L1B_APID_MAP[LID][0]
   else:
      return 'UNKNOWN'

def get_PCFTemplate(LID):
   if (L1A_APID_MAP.has_key(LID)):
      return L1A_APID_MAP[LID][2]
   elif (L1B_APID_MAP.has_key(LID)):
      return L1B_APID_MAP[LID][2]
   else:
      return 'UNKNOWN:'

def extractMetAPID(FName):
   PARTS = os.path.split(FName)
   rAPID = PARTS[1][5:8]
   return rAPID

def extractAPID(FName):
   fExt = os.path.splitext(os.path.basename(FName))[1]
   if ((fExt == '.PDS') or (fExt == '.EDS')):
      PARTS = os.path.split(FName)
      rAPID = PARTS[1][5:8]
   elif ((fExt != '.met') and (fExt != '.hdr')):
      PARTS = string.split(FName, '.')
      rAPID = PARTS[len(PARTS)-1]
   else:
      rAPID = ''
   return rAPID

def getGBADName(PktDir):
   CANDIDATES = os.listdir(os.path.join(PktDir, 'P1540957*'))
   return CANDIDATES

def getInstrumentPCFTemplate(Inst, Lvl):
   if (Lvl == 'L1A'):
      if (L1A_Instrument_Map.has_key(Inst)):
         return L1A_Instrument_Map[Inst]
   elif (Lvl == 'L1B'):
      if (L1B_Instrument_Map.has_key(Inst)):
         return L1B_Instrument_Map[Inst]
   else:
      return 'UNKNOWN'
   return 'UNKNOWN'


class APID:
   WORKING_DIR = ""
   PACKET_DIR = ""

   def __init__(self, value):
      self.APID = value		# APID for this class.
      self.pktName = ""		# data packet file name
      self.pktHdrName = ""	# data packet header file name
      self.PCFTemplate = ""	# PCF template to be used, based on APID
      self.ST=0.0		# start time for this APID's data packets
      self.ET=0.0		# end time for this APID's data packets
      self.START_STR = ""
      self.END_STR = ""
      self.L0_Files = []	# list of L0 data files for this APID
      self.L0_Meta = []		# list of L0 metadata files for this APID
      self.isEDU=0		# FALSE/TRUE acceptable values
      self.isValid=0		# One if there is a data file for this APID

   def __repr__(self):
      OutStr = 'APID:       ' + self.APID
      OutStr = OutStr + '\npktName:    ' + self.pktName
      OutStr = OutStr + '\npktHdrName: ' + self.pktHdrName
      OutStr = OutStr + '\nPCFTemplate:' + self.PCFTemplate
      OutStr = OutStr + '\nST:          %d' % self.ST
      OutStr = OutStr + '\nET:          %d' % self.ET
      OutStr = OutStr + '\nSTART_STR:   ' + self.START_STR
      OutStr = OutStr + '\nEND_STR:     ' + self.END_STR
      #OutStr = OutStr + '\nL0_Files:   ' + self.L0_Files
      #OutStr = OutStr + '\nL0_Meta:    ' + self.L0_Meta
      #OutStr = OutStr + '\nisEDU:      ' + self.isEDU
      #OutStr = OutStr + '\nisValid:    ' + self.isValid
      return OutStr


   def getAPID(self):
      return self.APID

   def getWorkingDir(self):
      if (self.isValid):
         return APID.WORKING_DIR
      else:
         return ''

   def setWorkingDir(self, value):
      APID.WORKING_DIR = value	# NOTE: sets working directory for all instances.

   def setPacketDir(self, value):
      APID.PACKET_DIR = value	# NOTE: sets packet directory for all instances.

   def getpktName(self):
      return self.pktName

   def setpktName(self,value):
      self.pktName = value

   def getpktHdrName(self):
      return self.pktHdrName

   def setpktHdrName(self,value):
      self.pktHdrName = value

   def getPCFTemplate(self):
      return self.PCFTemplate

   def setPCFTemplate(self):
      if (self.isValid):
         self.PCFTemplate = get_PCFTemplate(self.APID)

   def getStartTimeStr(self):
      return self.START_STR

   def setStartTimeStr(self,value):
      if (self.isValid):
         self.START_STR = value

   def getStartTime(self):
      return self.ST

   def setStartTime(self,value):
      if (self.isValid):
         self.ST = value

   def getEndTimeStr(self):
      return self.END_STR

   def setEndTimeStr(self, value):
      if (self.isValid):
         self.END_STR = value

   def getEndTime(self):
      return self.ET

   def setEndTime(self, value):
      if (self.isValid):
         self.ET = value

   # returns a list containing start and end times for the packet file processed.
   def extractHdrTimes(self):
      # extract and set the start time and end time strings.
      # requires that we read the header file we just generated, extract the first
      # and last time strings, and then dump the extra data.
      DATALINES = open(os.path.join(APID.WORKING_DIR, self.pktHdrName)).readlines()
      # Now read through the DATALINES from first -> last until you fine "TIME string="
      idx = 0
      STARTSTR=""
      while (STARTSTR == ""):
	 DATALINES[idx] = string.strip(DATALINES[idx])
         PARTS = string.split(DATALINES[idx], '=')
	 if (PARTS[0] == 'TIME string'):
	    STARTSTR = PARTS[1]
	    break
	 idx = idx + 1

      # Now read through the DATALINES from last -> first until you find "TIME string="
      DATALINES.reverse()
      idx = 0
      ENDSTR=""
      while (ENDSTR == ""):
	 DATALINES[idx] = string.strip(DATALINES[idx])
         PARTS = string.split(DATALINES[idx], '=')
	 if (PARTS[0] == 'TIME string'):
	    ENDSTR = PARTS[1]
	    break
	 idx = idx + 1
      # Set the values found into self
      # Flush the file buffer:
      DATALINES=[]
      # convert the UTC time string to unix time for start
      PARTS=[]
      STARTSTR = string.strip(STARTSTR)
      PARTS = string.split(STARTSTR, ' ')
      self.START_STR = PARTS[0]
      PARTS=[]
      ENDSTR = string.strip(ENDSTR)
      PARTS = string.split(ENDSTR, ' ')
      self.END_STR = PARTS[0]
      #print 'DEBUG(extractHdrTimes): Start time =',self.START_STR
      #print 'DEBUG(extractHdrTimes): End time   =',self.END_STR
      StTpl = airsTm.UTC2tm(self.START_STR)
      EndTpl = airsTm.UTC2tm(self.END_STR)
      self.ST = airsTm.unixTime(StTpl)
      self.ET = airsTm.unixTime(EndTpl)

   def genPktHdrs(self):
      if (self.isValid):
         # Check to see if we have any packets for our APID in the PACKET_DIR
	 HDR_CMD = 'run_scan_packets '
	 self.pktHdrName = self.pktName+'.hdr'
	 if (self.isEDU):
	    HDR_CMD = HDR_CMD + '-skip 20 '
	 HDR_CMD = HDR_CMD + os.path.join(APID.PACKET_DIR, self.pktName) + \
	           " > " + os.path.join(APID.WORKING_DIR, self.pktHdrName)
         RESULTS = os.popen(HDR_CMD).readlines()
         #print 'DEBUG(apid.py:genPktHdrs): ',HDR_CMD,'Results:',RESULTS
         # Now verify that the hdr file was generated and exists in WORKING_DIR
         if (not(os.path.exists(os.path.join(APID.WORKING_DIR, self.pktHdrName)))):
            print 'ERROR: attempt to generate header <',self.pktHdrName,"> FAILED."
	    self.isValid = 0
	    return
	 else:
            self.extractHdrTimes()

   def genL0Files(self):
      DateStr = airsTm.timehack(self.ST)
      RAWDIR = "/var/tmp/"+DateStr
      if (os.path.exists(RAWDIR)):
         if (os.path.exists(RAWDIR+"/Granule-000")):
	    for df in os.listdir(RAWDIR+"/Granule-000"):
	       os.remove (RAWDIR+"/Granule-000/"+df)
	 else:
            os.mkdir(RAWDIR+"/Granule-000")
      else:
         os.mkdir(RAWDIR)
         os.mkdir(RAWDIR+"/Granule-000")
      lnName="%s/Granule-000/level-0.apid%s.1.dat" % (RAWDIR, self.APID)
      #print 'RAWDIR:',RAWDIR
      srcName="%s/%s" % (self.PACKET_DIR, self.pktName)
      #print 'lnName:',lnName
      os.symlink(srcName, lnName)
      CMD = "%s %s %s %s %s 0 0 0 -1 20" % ('run_edos3_shift', DateStr, self.APID, self.WORKING_DIR, RAWDIR)
      #print 'DEBUG(genL0Files): CMD = ',CMD
      RESULTS = os.popen(CMD).readlines()
      #print ">",RESULTS

   def setPktFiles(self):
      if ((APID.WORKING_DIR != "") and (APID.PACKET_DIR != "")):
         # Check to see if we have any packets for our APID in the PACKET_DIR
	 CANDIDATES = os.listdir(APID.PACKET_DIR)
	 for FILE in CANDIDATES:
	    string.strip(FILE)
	    #print 'setPktFiles: checking <',FILE,'> as candidate'
	    if ((string.find (FILE, self.APID) != -1) and (string.find (FILE, "hdr$") == -1)):
	       # Ok, we found our APID in a file name, and the file isn't a header file
	       # Assign it to self.pktName, and check to see if it's an EDU format file.
	       self.pktName = FILE
	       #print 'setPktFiles: setting <',FILE,'> as pktName'
	       self.isEDU = airs_utils.eduTest (os.path.join(APID.PACKET_DIR, self.pktName))
	       self.isValid = 1
