#!/usr/bin/env python

#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

import os, sys, string, time
import airsTm, apid, esdt, airs_env

#########################################################################
# Given a set of EDS/PDS data files, generate the information set using	#
# l0.pl and collect the output in a list for subsequent processing.	#
#									#
# NOTE: This modules code cannot be accessed until esdt.gen_esdt_tbl()	#
# has been called in the main program body.				#
#########################################################################


METADATA = {'SHORTNAME': 'UNKNOWN',
            'LONGNAME' : 'UNKNOWN',
	    'PRODUCTIONDATETIME' : airsTm.ut2UTC(time.time()),
	    'RANGEENDINGDATE' : 'UNKNOWN',
	    'RANGEENDINGTIME' : 'UNKNOWN',
	    'RANGEBEGINNINGDATE' : 'UNKNOWN',
	    'RANGEBEGINNINGTIME' : 'UNKNOWN',
	    'HEADERFILENAME' : 'UNKNOWN',
	    'METADATANAME' : 'UNKNOWN',
	    'APID' : 'UNKNOWN' }

NULL_DATE = '1958-01-01T00:00:00.000000Z'
#########################################################################
# splitdate is a short utility function to split AIRS UTC format date	#
# strings into date and time portions for subsequent use.  See below.	#
#########################################################################
def splitdate(DATE):
   RTN = string.split(DATE, 'T')
   return RTN


#########################################################################
# genMet - given a set of EDS/PDS data file names, determine which is	#
# the control file (xxx00.[EP]DS) and extract the required data for the	#
# Level 0 metadata file.  Other information is derived from the environ	#
# ment, all collected data is place into the METADATA dictionar for use	#
# in generating the actual Level 0 metadata file.			#
#########################################################################
def genMet(DataSet):
   global METADATA

   # set should have two elements, [EP]1540*0.[PE]DS & [EP]1540*1.[PE]DS.
   # DO NOT ATTEMPT TO PROCESS THE *1 data file; that is the actual full data
   # set and will overload the system - and crash it!

   for idx in range (len(DataSet)):
      (Pth, Name) = os.path.split(DataSet[idx])
      (fName, fExt) = os.path.splitext(Name)
      if (fName[len(fName)-1] == '0'):
         CName = DataSet[idx]
   CMD_NAME = airs_env.getENV('AIRSDB')+'/bin/l0.pl'
   CMD = CMD_NAME +' -v %s' % (CName)
   RESULTS = os.popen(CMD).readlines()
   RESULTS2=[]
   HOLD = []
   HOLD_SIZE = 0
   DIGITS = '0123456789'
   LAST_ENTRY = len(RESULTS)-1
   for idx in range(len(RESULTS)):
      RESULTS[idx] = string.strip(RESULTS[idx])
	    
   idx = 0
   while idx < LAST_ENTRY:
      ENTRY = ''
      ch1 = RESULTS[idx][0]
      ch2 = RESULTS[idx+1][0]
      if ((ch1 in DIGITS) and (ch2 in DIGITS)):
         RESULTS2.append(RESULTS[idx])
	 idx = idx + 1
      else:
         ENTRY = RESULTS[idx] + RESULTS[idx+1]
	 RESULTS2.append(ENTRY)
	 idx = idx + 2

   for line in RESULTS2:
      #print '-%s-' % (line)
      if (string.find (line, '=>') > 0):
         PARTS = string.split(line, '=>')
         value = string.strip(PARTS[1])
         (tag, label) = string.split(PARTS[0], ':')
         string.strip (tag)
         string.strip (label)
         string.strip (value)
	 if (tag == '25-2'):
	    (PTH, fName) = os.path.split(value)
	    (bName, fExt) = os.path.splitext(fName)
	    if (bName[len(bName)-1] == '1'):
	       METADATA['METADATANAME'] = fName + '.met'
	       METNAME = fName + '.met'
	       #print 'Output metadata file name: %s'%(METNAME)
	    elif (bName[len(bName)-1] == '0'):
	       METADATA['HEADERFILENAME'] = bName
	 if ((tag == '25-4.2') and
	     (string.find(label, 'APID') > 0) and (value != '0')):
	    #print 'APID: -%s-' % (value)
	    METADATA['APID'] = value
	    iAPID = string.atoi(value)
	    METADATA['SHORTNAME'] = esdt.ESDT_TBL[iAPID].get_short()
	    METADATA['LONGNAME'] = esdt.ESDT_TBL[iAPID].get_desc()
         if ((tag == '25-4.3') and (value != NULL_DATE)):
            #print 'Start Time: %s' % (value)
	    DATE_TIME = splitdate(value)
	    METADATA['RANGEBEGINNINGDATE'] = DATE_TIME[0]
	    METADATA['RANGEBEGINNINGTIME'] = DATE_TIME[1]
         if ((tag == '25-4.4') and (value != NULL_DATE)):
            #print 'End Time: %s' % (value)
	    DATE_TIME = splitdate(value)
	    METADATA['RANGEENDINGDATE'] = DATE_TIME[0]
	    METADATA['RANGEENDINGTIME'] = DATE_TIME[1]

#########################################################################
# genMetadataFile: generate the actual level 0 metadata file.  Uses	#
# string substitution into the defined metadata template.		#
#########################################################################
def genMetadataFile(Template, Directory):
   INFILE = open(Template, 'r').readlines()
   OUT = open (os.path.join(Directory,METADATA['METADATANAME']), 'w')

   for item in INFILE:
      item = string.rstrip(item)
      for KEY in METADATA.keys():
         if (string.find(item, KEY) > 0):
	    TAG = '<%s>'%(KEY)
	    item = string.replace(item, TAG, METADATA[KEY])
      OUT.write ('%s\n'%(item))
      #print 'Writing: %s'%(item)
   OUT.close()

#########################################################################
