
#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

# FileName: meta.py
#########################################################################
# Description:  Python module which parses an AIRS metadata file and	#
# makes available information extracted from it.  Initial version is	#
# limited to extracting object lists from the file, and primary		#
# emphasis is on extracting the start data and end date from there.	#
# In the longer run the goal is to develope an object data structure	#
# which can contain the group collection of the metadata file.  This	#
# is a recursive structure, consisting of an exterior group, such as	#
# "INVENTORYMETADATA", which contains nested subgroups and objects.	#
# Each subgroup, in turn, may also contain more nested levels of groups	#
# and objects.  Objects are not recursive in structure; an object is an	#
# is an atomic element, containing no other objects or groups.		#
#########################################################################

import string

OL={}

class meta_Object:
   def __init__(self,Name):
      self.name = Name
      self.value = ""
      self.type = "STRING"
      self.nval = 1

   def setVal(self, Value):
      self.value = Value

   def getVal(self):
      return self.value

   def settype(self, Type):
      self.type = Type

   def gettype(self):
      return self.type

   def setnum_val(self, NVal):
      self.nval = NVal

   def getnum_val(self):
      return self.nval

def getObjectValue(MetFileName, OBJ):
   MET_DATA = open(MetFileName, "r").readlines()
   EI=0
   SI=0
   Size = len(MET_DATA)

   for idx in range (len(MET_DATA)):
      MET_DATA[idx] = string.strip(MET_DATA[idx])
      MET_DATA[idx] = string.replace(MET_DATA[idx], '\"', '')
      PARTS = string.split(MET_DATA[idx], '=')
      if (len(PARTS) == 1):
         continue
      PARTS[0] = string.strip(PARTS[0])
      PARTS[1] = string.strip(PARTS[1])
      if ((PARTS[0] == 'END_OBJECT') and (PARTS[1] == OBJ)):
         EI=idx
      if ((PARTS[0] == 'OBJECT') and (PARTS[1] == OBJ)):
         SI=idx
      if ((SI >0) and (EI > 0)):
         break

   for idx in range (SI, EI):
      PARTS = string.split(MET_DATA[idx], '=')
      PARTS[0] = string.strip(PARTS[0])
      PARTS[1] = string.strip(PARTS[1])
      if ((PARTS[0] == 'VALUE') or (PARTS[0] == 'Value') or (PARTS[0] == 'value')):
         return PARTS[1]
      if ((PARTS[0] == 'END_OBJECT') and (PARTS[1] == OBJ)):
         break


def getDateInfo(MetFileName):
   MET_DATA = open(MetFileName, "r").readlines()

   EI=0
   SI=0
   for idx in range (len(MET_DATA)):
      MET_DATA[idx] = string.replace(MET_DATA[idx], '\"', '')
      PARTS = string.split(MET_DATA[idx], '=')
      if (len(PARTS) == 1):
         break
      PARTS[0] = string.strip(PARTS[0])
      PARTS[1] = string.strip(PARTS[1])
      if ((PARTS[0] == 'END_GROUP') and (PARTS[1] == 'RangeDateTime')):
         EI=idx
      if ((PARTS[0] == 'GROUP') and (PARTS[1] == 'RangeDateTime')):
         SI=idx
      if (SI and EI):
         break

   for idx in range (SI, EI):
      PARTS = string.split(MET_DATA[idx], '=')
      PARTS[0] = string.strip(PARTS[0])
      PARTS[1] = string.strip(PARTS[1])
      if (PARTS[0] == 'OBJECT'):
         CURR_OBJ = PARTS[1]
         OL[PARTS[1]] = meta_Object(PARTS[1])
      if ((PARTS[0] == 'VALUE') or (PARTS[0] == 'Value') or (PARTS[0] == 'value')):
         OL[CURR_OBJ].setVal(PARTS[1])
      if ((PARTS[0] == 'TYPE') or (PARTS[0] == 'Type') or (PARTS[0] == 'type')):
         OL[CURR_OBJ].settype(PARTS[1])
      if ((PARTS[0] == 'NUM_VAL') or (PARTS[0] == 'Num_Val') or (PARTS[0] == 'num_val')):
         OL[CURR_OBJ].setnum_val(PARTS[1])
      if (PARTS[0] == 'END_OBJECT'):
         CURR_OBJ = ''
      if ((PARTS[0] == 'END_GROUP') and (PARTS[1] == 'RangeDateTime')):
         break

def getStartDate():
   StartDate = '%sT%s' % (OL["RangeBeginningDate"].getVal(),OL["RangeBeginningTime"].getVal())
   return StartDate

def getEndDate():
   EndDate = OL["RangeEndingDate"].getVal()+'T'+OL["RangeEndingTime"].getVal()
   return EndDate
