
#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

import string, pcf, os, apid, esdt

class PCF:
   APID_SET = {}
   def __init__(self, pcftemplatename):
      # verify the availability of the pcf template, and load template
      # first to list, then to dictionary based on the line # * 100 as key.
      self.ProcLevel = 'L1A'
      L1B_POSN = string.find(pcftemplatename, 'L1B')
      TypeVal=type(pcftemplatename)
      if (L1B_POSN >= 0):
         self.ProcLevel = 'L1B'
      self.pcfTbl={}
      self.LIDTbl={}
      self.replaceTbl={}
      self.APID_NameSet={}
      pcfVFlds = {}
      pcfLCnt={}
      pcflist=pcf.loadPCFTemplate(pcftemplatename, pcfVFlds, pcfLCnt)
      for idx in range (len(pcflist)):
         key = idx*100
	 self.pcfTbl[key] = pcflist[idx]
	 tLID = pcflist[idx].getLID()
	 if (self.LIDTbl.has_key(tLID)):
	    self.LIDTbl[tLID].append(key)
	 else:
	    self.LIDTbl[tLID] = []
	    self.LIDTbl[tLID].append(key)
      for x in apid.L1A_APIDS:
         self.APID_SET[x] = 0
      for x in apid.L1B_APIDS:
         self.APID_SET[x] = 0



   def getPCFLine(self,LID):
      if (self.LIDTbl.has_key(LID)):
         Index=self.LIDTbl[LID][0]
         PCF_Str = '%s' % (self.pcfTbl[Index])
         return PCF_Str
      else:
         return ""

   def setFldVal(self, LID, Field, Value):
      if (self.LIDTbl.has_key(LID)):
         Index=self.LIDTbl[LID][0]
	 self.pcfTbl[Index].repFld(self.pcfTbl[LID], Field, Value)

   def setRepTblVal(self, STR, REP):
      self.replaceTbl[STR] = REP

   def doReplacements(self):
      # requires that we iterate through all pcf lines in the table, check for
      # each value in the replacement table, and substitute that field.
      keylist = self.pcfTbl.keys()
      keylist.sort()
      for i in (keylist):
         for key in (self.replaceTbl.keys()):
	    self.pcfTbl[i].repStr(key, self.replaceTbl[key])


   def getLIDCnt(self, lid):
      tLID = int(lid)
      if (self.LIDTbl.has_key(tLID)):
         return len(self.LIDTbl[tLID])
      else:
	 return 0

   def extractBasename(self, Name, nList):
      Name = string.strip(Name)
      NameSet=[]
      BName=os.path.splitext(os.path.basename(Name))[0][:-1]
      for idx in range (len(nList)):
         X = string.find(nList[idx], BName)
	 if (X >= 0):
	    NameSet.append(nList[idx])
      for idx in range(len(NameSet)):
         X =nList.index(NameSet[idx])
	 del nList[X]
      return NameSet

   def repLine(self, LID, REPSTR):
      if (self.LIDTbl.has_key(LID)):
         NewLine = pcf.PCF_LINE(REPSTR)
	 Index = self.LIDTbl[LID][0]
	 self.pcfTbl[Index] = NewLine
   
   def genLIDLineSet(self, LID, DataSet, Fld):
      # PROBLEM:  in processing the sample DB dataset we received, it
      # included a group of ".mdx" files.  These present a processing
      # problem and must be removed from the list.
      lSize = len(DataSet)
      # first locate the metadata file entry & delete from DataSet
      DelList = []
      for idx in range(len(DataSet)):
         fExt = os.path.splitext(os.path.basename(DataSet[idx]))[1]
	 if (fExt == '.mdx'):
	    DelList.append(idx)
      if (len(DelList) > 0):
         DelList.reverse()
         for idx in DelList:
            #print 'DEBUG(genLIDLineSet):deleting->',DataSet[idx]
            del DataSet[idx]
      DelList = []
      for idx in range(len(DataSet)):
         BName = os.path.splitext(os.path.basename(DataSet[idx]))[1]
	 if (BName == '.met'):
	    MetName = DataSet[idx]
	    DelList.append(idx)
      if (len(DelList) > 0):
         DelList.reverse()
         for idx in DelList:
            del DataSet[idx]
      if (self.LIDTbl.has_key(LID)):
         Cnt = len(DataSet)
	 Index = self.LIDTbl[LID][0]
	 LidLine=self.pcfTbl[Index].getLine()
	 LineList = []
	 versFld = self.pcfTbl[Index].getVersionField()
	 MetFld = versFld - 1
	 LGIDFld = versFld - 2
	 for idx in DataSet:
	    pline = pcf.PCF_LINE(LidLine)
	    pline.repFld(Fld, string.strip(idx))
	    pline.repFld(MetFld, string.strip(MetName))
	    pline.repFld(LGIDFld, esdt.ESDT_TBL[LID].gen_LGID_Name(string.strip(idx)))
	    pline.repFld(versFld, `Cnt`)
	    Cnt = Cnt - 1
	    LineList.append(pline)
	 # we now have a list of entries to be made to the pcf file.
	 for item in LineList:
	    self.pcfTbl[(Index-lSize)] = item
	    lSize = lSize-1
	 del self.pcfTbl[Index]
      else:
         print 'ERROR in processing line expansion:',LID,'not found'
      return LineList

   def procAPIDList(self,APID,nList):
      nList.sort()
      LID_Set=[]
      DataSet = nList
      for x in range(len(DataSet)):
         DataSet[x] = string.strip(DataSet[x])
      tLID = APID
      nLID = '9'+APID
      LID_Set.append(int(tLID))
      LID_Set.append(int(nLID))
      for I in LID_Set:
         if self.APID_SET[`I`]:
	    print 'APID %d already in use, procAPID quitting.' % (I)
	    return
	 else:
	    self.APID_SET[`I`] = 1
      Fld = 1
      Cnt = self.getLIDCnt(tLID)
      if (Cnt > 1):
         print 'ERROR: multiple APID %s lines in PCF.  Cannot process.' % (tLID)
      APID_Set=[]
      for I in LID_Set:
	 if (len(DataSet) > 0):
            APID_Set = self.extractBasename(DataSet[0], DataSet)
	 else:
	    APID_Set = []
	 # suspicion: this line should be deindentd by 3.
	 if (len(APID_Set) > 0):
	    LineSet = self.genLIDLineSet(I, APID_Set, Fld)

   def procAPID(self, Param, FName):
      LID_Set = []
      # Called from expandMultiFile if specified LID is an APID.  May be
      # called directly if that is appropriate.  Can eliminate some of the
      # verification as we already know that the APID exists as a LID in
      # the current PCF.
      DataSet = open(FName, 'r').readlines()
      for x in range(len(DataSet)):
         DataSet[x] = string.strip(DataSet[x])
      Param = string.replace(Param, 'multi', '')
      LData = string.split(Param, 'F')
      tLID = LData[0]
      nLID = '9'+LData[0]
      LID_Set.append(int(LData[0]))
      LID_Set.append(int(nLID))
      for I in LID_Set:
         if self.APID_SET[`I`]:
	    print 'APID %d already in use, procAPID quitting.' % (I)
	    return
	 else:
	    self.APID_SET[`I`] = 1
      Fld = int(LData[1])
      Cnt = self.getLIDCnt(tLID)
      if (Cnt > 1):
         print 'ERROR: Multiple APID %s lines in PCF.  Cannot process.' % (tLID)
      for I in LID_Set:
         APID_Set = self.extractBasename(DataSet[0], DataSet)
	 if (len(APID_Set) > 0):
	    LineSet = self.genLIDLineSet(I, APID_Set, Fld)

   def expandMultiList(self, LID, DataSet):
      lSize = len(DataSet)
      tLID = `LID`
      if (self.LIDTbl.has_key(LID)):
         Cnt = self.getLIDCnt(tLID)
	 if (Cnt > 1):
	    print 'ERROR in processing multifile input list for LID=',LID
	    return
	 Index=self.LIDTbl[LID][0]
	 lSize = len(DataSet)
	 for item in DataSet:
	    self.pcfTbl[(Index-lSize)] = item
	    lSize = lSize - 1
	 del self.pcfTbl[Index]

   def expandMultiFile(self, Param, FName):
      # Param is of the form multi<LID>F<F#>.  Fname is the name of the file
      # holding the parameter values to be added to the pcf file.
      # NOTE:  current procedure requires that if there is only one line with
      # the specified LID then that line will be copied the number of times
      # necessary to accomodate all lines in the multifile for the specified
      # field.  If more than one copy exists, the the input will be discarded.
      DataSet = open(FName, 'r').readlines()
      for x in range(len(DataSet)):
         DataSet[x] = string.strip(DataSet[x])
      #lSize = len(DataSet)
      Param = string.replace(Param, 'multi', '')
      LData = string.split(Param, 'F')
      LID = int(LData[0])
      tLID = LData[0]
      if (tLID in apid.L1A_APIDS):
         self.procAPID(Param, FName)
	 return
      Fld = self.getLIDCnt(tLID)
      Cnt = self.getLIDCnt(tLID)
      if (Cnt > 1):
         print 'ERROR in processing multifile:',Param,'=',FName
	 return
      LineList = self.genLIDLineSet(LID, DataSet, Fld)

   def replaceField(self, lid, val, Fld):
      Cnt = self.getLIDCnt(lid)
      if (Cnt > 0):
         for idx in self.LIDTbl[string.atoi(lid)]:
	    self.pcfTbl[idx].repFld(Fld, val)

   def clearCommentFlag(self, lid):
      # Many of the LID's, especially the APID LIDs, are commented out in 
      # the templates.  Be sure that any referenced LID has had the comment
      # flag cleared.
      Cnt = self.getLIDCnt(lid)
      if (Cnt > 0):
         for idx in self.LIDTbl[lid]:
	    self.pcfTbl[idx].clearComment()

   def setInstrument(self, Inst):
      self.instrument = Inst

   def setValidAPIDSet(self):
      INSTRUMENT_FAILURE = 'Specified instrument does not exist'
      INSTRUMENT_NOT_DEFINED = 'Instrument not defined'
      LVL = self.ProcLevel
      if (self.instrument != ''):
         if (self.instrument == 'AIRS'):
	    if (LVL == 'L1A'):
	       self.validAPIDSet = apid.L1A_AIRS_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1A_AIRS_APIDS
	    else:
	       self.validAPIDSet = apid.L1B_AIRS_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1B_AIRS_APIDS
	 elif (self.instrument == 'AMSU'):
	    if (LVL == 'L1A'):
	       self.validAPIDSet = apid.L1A_AMSU_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1A_AMSU_APIDS
	    else:
	       self.validAPIDSet = apid.L1B_AMSU_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1B_AMSU_APIDS
	 elif (self.instrument == 'HSB'):
	    if (LVL == 'L1A'):
	       self.validAPIDSet = apid.L1A_HSB_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1A_HSB_APIDS
	    else:
	       self.validAPIDSet = apid.L1B_HSB_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1B_HSB_APIDS
	 elif (self.instrument == 'VIS'):
	    if (LVL == 'L1A'):
	       self.validAPIDSet = apid.L1A_HSB_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1A_HSB_APIDS
	    else:
	       self.validAPIDSet = apid.L1B_HSB_APIDS
	       self.PrimaryAPIDSet = apid.PRIMARY_L1B_HSB_APIDS
	 else:
	    raise INSTRUMENT_FAILURE, self.instrument
      else:
         raise INSTRUMENT_NOT_DEFINED

   def genAPIDDataSet(self, SRCDIR):
      EDSNamePattern = os.path.join(SRCDIR, 'E1540XXX*.EDS')
      PDSNamePattern = os.path.join(SRCDIR, 'P1540XXX*.PDS')
      Candidates = os.listdir(SRCDIR)
      DelMe = []

      for idx in range(len(Candidates)):
         if (string.find(Candidates[idx], '1540') >= 0):
	    self.setRepTblVal('must-set-pds-directory', SRCDIR)
	    PARTS = os.path.split(Candidates[idx])
	    cAPID = PARTS[1][5:8]
            fExt = os.path.splitext(Candidates[idx])[1]
	    if ((cAPID in self.validAPIDSet) and (fExt != '.hdr')):
	       if (self.APID_NameSet.has_key(cAPID)):
	          self.APID_NameSet[cAPID].append(Candidates[idx])
	       else:
		  self.APID_NameSet[cAPID] = []
	          self.APID_NameSet[cAPID].append(Candidates[idx])

      for idx in self.PrimaryAPIDSet:
         #print 'DEBUG(genAPIDDataSet): idx = ',idx
         if (self.APID_NameSet.has_key(idx)):
	    self.procAPIDList(idx, self.APID_NameSet[idx])

   def setMetafiles(self):
      keylist = self.pcfTbl.keys()
      keylist.sort()
      for i in (keylist):
         self.pcfTbl[i].setMetafileName()

   def writePCF(self, OutName):
      PCF_File = open(OutName, 'w')
      keylist = self.pcfTbl.keys()
      keylist.sort()
      for i in (keylist):
         self.pcfTbl[i].setURField()
         Item = '%s' % (self.pcfTbl[i])
	 Line = '%s\n' % (string.strip(Item))
	 PCF_File.write(Line)
      PCF_File.close()

   def __repr__(self):
      keylist=self.pcfTbl.keys()
      keylist.sort()
      for i in (keylist):
	 Line = '%s' % (self.pcfTbl[i])
         print string.strip(Line)
      return ''

