#!/usr/bin/env python

#------------------------------------------------------------------------------
#
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------

import os, sys, meta, fnmatch, re, string, shutil, airs_utils

# use of shutil.copy2 is a temporary fix to support requirements for
# direct broadcast and related immediate EDS file processing at JPL.
# Longer term solution is to complete L1B processing & copy all outputs
# back to the source EDS directory whos path is specified by the env
# variable AIRSDB_PACKETS.  ToPath should be given as $AIRSDB_PACKETS.
def renameFiles (Path, ToPath='', PROC='L1A'):
   if (PROC == 'L1A'):
      Pattern = fnmatch.translate('L1A-*.met')
   else:
      Pattern = fnmatch.translate('L1B-*.met')
   Candidates = os.listdir(Path)
   MetFiles=[]
   for idx in range(len(Candidates)):
      Candidates[idx] = string.strip (Candidates[idx])
      if re.match(Pattern, Candidates[idx]):
         MetFiles.append(Candidates[idx])
   Candidates =[]
   

   for MetaFileName in MetFiles:
      MetName = os.path.join(Path, MetaFileName)
      FileName = os.path.splitext(MetaFileName)[0]
      TempName = meta.getObjectValue(MetName, 'LOCALGRANULEID')
      GranIDName = '%s' % (TempName)
      GranIDMetName = '%s.met' % (GranIDName)
      # NOTE: need to check to see if ToPath protections will allow
      # us to copy files there.  If not, then set ToPath to Path
      # and leave the files there.  A warning message should be 
      # displayed.
      if (ToPath != ''):
         if not (airs_utils.canWriteTo(ToPath)):
	    ToPath = Path
      if (ToPath == ''):
	 mSrc = os.path.join(Path,MetaFileName)
	 mDst = os.path.join(Path,GranIDMetName)
	 Src = os.path.join(Path,FileName)
	 Dst = os.path.join(Path,GranIDName)
      else:
	 mSrc = os.path.join(Path,MetaFileName)
	 mDst = os.path.join(ToPath,GranIDMetName)
	 Src = os.path.join(Path,FileName)
	 Dst = os.path.join(ToPath,GranIDName)
      shutil.copy2 (Src, Dst)
      shutil.copy2 (mSrc, mDst)

# Locate all log files in the SRC directory, move them to the DST
# directory.
def moveLogs(SRC, DST):
   if (airs_utils.canWriteTo(DST)):
      LogPattern = fnmatch.translate("Log[SRU]*")
      FileList = os.listdir(SRC)
      LogList=[]
      for name in FileList:
         name = string.strip(name)
         if re.match(LogPattern, name):
            LogList.append(name)
      FileList=[]
      for name in LogList:
         src = os.path.join(SRC, name)
         dst = os.path.join(DST, name)
         shutil.copy2(src, dst)
